
/*
  To drive a 4 digit VFD controlled by a string of 74HC595 chips or other SPI-like interface.
  See VFD1.ino for hardware details
*/
#ifndef VFD_H //include guard
#define VFD_H

#ifdef VFD_DRV // definions only relevent to driver

   // Arduino PIN definitions
   // set pin 10 as the slave select for the '595 chain:
   const int VFDslaveSelectPin = 10;

   // set pin 9 as the master dimmer for the ULN2803 chips. 
   // MUST be a PWM capable pin
   const int VFDmdimPWM = 9;

   // display interface characteristics

  #define VFD_SPI_SET 100000, MSBFIRST, SPI_MODE0 // MODE 0, at 100KHz
  
   // comment out if not inverted
   #define VFD_INVERTED // ULN2803 inverts signal.  LOW segment + grid is lit. 

   // Uncomment one VFDSCRAMBLE only or write your own VFDSCRAMBLE 
   // macro for non-standard segment orders.

   #define VFD_SCRAMBLE(x) x // Dont scramble
         
   // Reverse (gfedcba) seg. order 
   //#define VFD_SCRAMBLE(x) ((x&64U)>>6|(x&32U)>>4  \
   //         |(x&16U)>>2|(x&8U)|(x&4U)<<2        \
   //         |(x&2U)<<4|(x&1U)<<6)
   
   #define VFD_DIGMAP 0b00000001, 0b00000010, 0b00000100, 0b00001000, 0b00010000, 0b00100000, 0b01000000, 0b10000000  //Alter digit mapping and total number here
   #define VFD_MUXRATE 255 //3ms per digit   
   #define VFD_BRIGHT 250 // 0 to 255

#endif //VFD_DRV

//Symbol to digit mapping - all in high bit
enum{
   VFD_SYMB_A=0,
   VFD_SYMB_COOK,
   VFD_SYMB_DOT,
   VFD_SYMB_BELL,
   VFD_SYMBBIT=128
};

extern const unsigned char VFDfont[]; //  7 segment HEX font
extern const unsigned char VFDdigits; // No. of digits
extern volatile unsigned char VFDdispBuff[]; // display buffer

#define VFDdigitWrite(digit,segments)   (VFDdispBuff[digit]=segments) //Display 1 digit
void VFDsetup(); //Setup VFD

#endif //include guard
