/*
  To drive a N digit VFD controlled by a string of 74HC595 chips or other SPI-like interface.
*/

#include <SPI.h>
#define VFD_DRV
#include "VFD.h"

// C source for 7 segment HEX font
// Bin patterns entered in normal 
// segment order: abcdefg
const unsigned char VFDfont[]={
   VFD_SCRAMBLE( 0b1111110 ), // '0'
   VFD_SCRAMBLE( 0b0110000 ), // '1'
   VFD_SCRAMBLE( 0b1101101 ), // '2'
   VFD_SCRAMBLE( 0b1111001 ), // '3'
   VFD_SCRAMBLE( 0b0110011 ), // '4'
   VFD_SCRAMBLE( 0b1011011 ), // '5'
   VFD_SCRAMBLE( 0b1011111 ), // '6'
   VFD_SCRAMBLE( 0b1110000 ), // '7'
   VFD_SCRAMBLE( 0b1111111 ), // '8'
   VFD_SCRAMBLE( 0b1110011 ), // '9'
   VFD_SCRAMBLE( 0b1110111 ), // 'A'
   VFD_SCRAMBLE( 0b0011111 ), // 'B'
   VFD_SCRAMBLE( 0b1001110 ), // 'C'
   VFD_SCRAMBLE( 0b0111101 ), // 'D'
   VFD_SCRAMBLE( 0b1001111 ), // 'E'
   VFD_SCRAMBLE( 0b1000111 )  // 'F'
};
// End Font

static const unsigned char digbit[]={VFD_DIGMAP};  //Digit select mapping
const unsigned char VFDdigits=sizeof(digbit);

volatile unsigned char VFDdispBuff[sizeof(digbit)];

// Support functions

void VFDsetup(){
   
   // Setup master dimmer
     pinMode(VFDmdimPWM, OUTPUT);
   #ifdef VFD_INVERTED
     analogWrite(VFDmdimPWM, 255-VFD_BRIGHT); // Set 1/4 brightness
   #else  
      analogWrite(VFDmdimPWM, VFD_BRIGHT); // Set 1/4 brightness
   #endif
   
   // set the slaveSelectPin as an output:
   pinMode(VFDslaveSelectPin, OUTPUT);
   // initialize SPI:
   SPI.begin();

   // Timer0 is already used for millis() - we'll just interrupt as
   // soon as possible after the Timer 0 overfloow ISR and call the 
   // "Compare A" function below
   OCR0A = 0;
   TIMSK0 |= _BV(OCIE0A);
}

// Timer 0 compare interrupt is called once a millisecond, 
SIGNAL(TIMER0_COMPA_vect) 
{
   static unsigned char digit;

   #ifdef VFD_MUXRATE  //Only decimate if required
   static unsigned char decimate;

   if (!decimate--){  // Payload fires 1 of N
      decimate=VFD_MUXRATE-1; 
   #endif

      // acquire the SPI peripheral
      SPI.beginTransaction(SPISettings(VFD_SPI_SET));
      // take the SS pin low to select the chip:
      digitalWrite(VFDslaveSelectPin, LOW);
      // send in the address and value via SPI:
      #ifdef VFD_INVERTED
         SPI.transfer16(~digbit[digit]);
         SPI.transfer(~VFDdispBuff[digit]);
      #else
         SPI.transfer16(digbit[digit]);
         SPI.transfer(VFDdispBuff[digit]);
      #endif
      // take the SS pin high to de-select the chip:
      digitalWrite(VFDslaveSelectPin, HIGH);
         // and release the SPI periperal
      SPI.endTransaction();
   
      if(!digit--) digit=VFDdigits; //set next digit
   #ifdef VFD_MUXRATE   
   }
   #endif
}
