/*
  To drive a 4 digit VFD controlled by a string of 74HC595 chips or other SPI-like interface.
  See VFD1.ino for hardware details
*/


#include <Arduino.h>
// List of extra headers to include - must copy to .ino
#include <SPI.h>
#include "VFD.h"

#define ONE_SEC 1000 // millis in one second.

// Globals
const char btnMode=3;
const char btnSet=2;

unsigned long BCDtime=0;

// Initialisation
void setup() {

   init();  //Arduino system initialisation
   VFDsetup(); //VFD initialisation.
   
   pinMode(btnMode, INPUT_PULLUP);
   pinMode(btnSet, INPUT_PULLUP);
}

void fixHMS(){
   //Count in sexagesimal BCD by bumping digits
   if((BCDtime&0x00000FUL)>=0x00000AUL) BCDtime+=0x000006UL; //Bump secs units
   if((BCDtime&0x0000F0UL)>=0x000060UL) BCDtime+=0x0000A0UL; //Bump secs tens
   if((BCDtime&0x000F00UL)>=0x000A00UL) BCDtime+=0x000600UL; //Bump mins units
   if((BCDtime&0x00F000UL)>=0x006000UL) BCDtime+=0x00A000UL; //Bump mins tens
   if((BCDtime&0x0F0000UL)>=0x0A0000UL) BCDtime+=0x060000UL; //Bump hours units
   if((BCDtime&0xFF0000UL)>=0x240000UL) BCDtime+=0xDC0000UL; //Bump hours at 24
}

int main() {

   unsigned long t;
   unsigned char d,s;
   unsigned long ms;
   char cbuff[4];

   setup(); // initialisation
   ms=millis();
 
   while(1){ // Superloop  

      if(digitalRead(btnSet)){
         if((millis()-ms)>ONE_SEC){
            ms+=ONE_SEC;
            BCDtime++;
         }
      }else{
         ms=millis();
         BCDtime+=0x000100;
         BCDtime&=0xFFFF00;
      }
      fixHMS();
      t=BCDtime;
      for(unsigned char dig=0; dig<VFDdigits; dig++){
         // Extract a digit
         d=(t>>(digitalRead(btnMode)?20:12)) & 0x0F;
         t<<=4;
         // Convert to segments
         s=VFDfont[d];
         if((dig==VFD_SYMB_DOT) && (BCDtime&1)) s|=VFD_SYMBBIT;
         // Output a segment pattern
         VFDdigitWrite(dig, s); 
      }
      delay(50);
   } //endWhile(1)
}//endFn main

